mirror of
https://github.com/NVIDIA/cuda-samples.git
synced 2026-10-13 00:08:26 +08:00
CUDA 13.2 samples update (#432)
- Added Python samples for CUDA Python 1.0 release - Renamed top-level `Samples` directory to `cpp` to accommodate Python samples.
This commit is contained in:
18
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/.vscode/c_cpp_properties.json
vendored
Normal file
18
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/.vscode/c_cpp_properties.json
vendored
Normal file
@@ -0,0 +1,18 @@
|
||||
{
|
||||
"configurations": [
|
||||
{
|
||||
"name": "Linux",
|
||||
"includePath": [
|
||||
"${workspaceFolder}/**",
|
||||
"${workspaceFolder}/../../../Common"
|
||||
],
|
||||
"defines": [],
|
||||
"compilerPath": "/usr/local/cuda/bin/nvcc",
|
||||
"cStandard": "gnu17",
|
||||
"cppStandard": "gnu++14",
|
||||
"intelliSenseMode": "linux-gcc-x64",
|
||||
"configurationProvider": "ms-vscode.makefile-tools"
|
||||
}
|
||||
],
|
||||
"version": 4
|
||||
}
|
||||
7
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/.vscode/extensions.json
vendored
Normal file
7
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/.vscode/extensions.json
vendored
Normal file
@@ -0,0 +1,7 @@
|
||||
{
|
||||
"recommendations": [
|
||||
"nvidia.nsight-vscode-edition",
|
||||
"ms-vscode.cpptools",
|
||||
"ms-vscode.makefile-tools"
|
||||
]
|
||||
}
|
||||
@@ -0,0 +1,54 @@
|
||||
cmake_minimum_required(VERSION 3.20)
|
||||
|
||||
list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/Modules")
|
||||
|
||||
project(EGLStream_CUDA_CrossGPU LANGUAGES C CXX CUDA)
|
||||
|
||||
find_package(CUDAToolkit REQUIRED)
|
||||
|
||||
set(CMAKE_POSITION_INDEPENDENT_CODE ON)
|
||||
|
||||
set(CMAKE_CUDA_ARCHITECTURES 75 80 86 87 89 90 100 110 120)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -Wno-deprecated-gpu-targets")
|
||||
if(ENABLE_CUDA_DEBUG)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -G") # enable cuda-gdb (may significantly affect performance on some targets)
|
||||
else()
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -lineinfo") # add line information to all builds for debug tools (exclusive to -G option)
|
||||
endif()
|
||||
|
||||
# Include directories and libraries
|
||||
include_directories(../../../Common)
|
||||
|
||||
find_package(EGL)
|
||||
|
||||
if(CMAKE_SYSTEM_NAME STREQUAL "Linux")
|
||||
# Source file
|
||||
if(${EGL_FOUND})
|
||||
# Add target for EGLStream_CUDA_CrossGPU
|
||||
add_executable(EGLStream_CUDA_CrossGPU cuda_consumer.cpp cuda_producer.cpp eglstrm_common.cpp kernel.cu main.cpp)
|
||||
|
||||
target_compile_options(EGLStream_CUDA_CrossGPU PRIVATE $<$<COMPILE_LANGUAGE:CUDA>:--extended-lambda>)
|
||||
|
||||
target_compile_features(EGLStream_CUDA_CrossGPU PRIVATE cxx_std_17 cuda_std_17)
|
||||
|
||||
set_target_properties(EGLStream_CUDA_CrossGPU PROPERTIES CUDA_SEPARABLE_COMPILATION ON)
|
||||
|
||||
target_include_directories(EGLStream_CUDA_CrossGPU PUBLIC
|
||||
${EGL_INCLUDE_DIR}
|
||||
${CUDAToolkit_INCLUDE_DIRS}
|
||||
)
|
||||
|
||||
target_link_libraries(EGLStream_CUDA_CrossGPU
|
||||
${EGL_LIBRARY}
|
||||
CUDA::cuda_driver
|
||||
)
|
||||
else()
|
||||
message(STATUS "EGL not found - will not build sample 'EGLStream_CUDA_CrossGPU'")
|
||||
endif()
|
||||
else()
|
||||
message(STATUS "Will not build sample EGLStream_CUDA_CrossGPU - requires Linux OS")
|
||||
endif()
|
||||
|
||||
# Include installation configuration
|
||||
include(${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/InstallSamples.cmake)
|
||||
setup_samples_install()
|
||||
@@ -0,0 +1,39 @@
|
||||
# EGLStream_CUDA_CrossGPU - EGLStream_CUDA_CrossGPU
|
||||
|
||||
## Description
|
||||
|
||||
Demonstrates CUDA and EGL Streams interop, where consumer's EGL Stream is on one GPU and producer's on other and both consumer-producer are different processes.
|
||||
|
||||
## Key Concepts
|
||||
|
||||
EGLStreams Interop
|
||||
|
||||
## Supported SM Architectures
|
||||
|
||||
[SM 5.0 ](https://developer.nvidia.com/cuda-gpus) [SM 5.2 ](https://developer.nvidia.com/cuda-gpus) [SM 5.3 ](https://developer.nvidia.com/cuda-gpus) [SM 6.0 ](https://developer.nvidia.com/cuda-gpus) [SM 6.1 ](https://developer.nvidia.com/cuda-gpus) [SM 7.0 ](https://developer.nvidia.com/cuda-gpus) [SM 7.2 ](https://developer.nvidia.com/cuda-gpus) [SM 7.5 ](https://developer.nvidia.com/cuda-gpus) [SM 8.0 ](https://developer.nvidia.com/cuda-gpus) [SM 8.6 ](https://developer.nvidia.com/cuda-gpus) [SM 8.7 ](https://developer.nvidia.com/cuda-gpus) [SM 8.9 ](https://developer.nvidia.com/cuda-gpus) [SM 9.0 ](https://developer.nvidia.com/cuda-gpus)
|
||||
|
||||
## Supported OSes
|
||||
|
||||
Linux
|
||||
|
||||
## Supported CPU Architecture
|
||||
|
||||
x86_64, armv7l
|
||||
|
||||
## CUDA APIs involved
|
||||
|
||||
### [CUDA Driver API](http://docs.nvidia.com/cuda/cuda-driver-api/index.html)
|
||||
cuDeviceGetName, cuEGLStreamConsumerReleaseFrame, cuEGLStreamConsumerConnect, cuEGLStreamConsumerDisconnect, cuCtxPushCurrent, cuEGLStreamProducerReturnFrame, cuStreamCreate, cuEGLStreamProducerPresentFrame, cuMemFree, cuGraphicsResourceGetMappedEglFrame, cuInit, cuMemcpyHtoD, cuDeviceGet, cuEGLStreamConsumerAcquireFrame, cuEGLStreamProducerDisconnect, cuEGLStreamProducerConnect, cuDeviceGetAttribute, cuCtxSynchronize, cuMemAlloc, cuCtxPopCurrent, cuCtxCreate
|
||||
|
||||
### [CUDA Runtime API](http://docs.nvidia.com/cuda/cuda-runtime-api/index.html)
|
||||
cudaMemcpy, cudaMalloc, cudaProducerPresentFrame, cudaFree, cudaGetErrorString, cudaConsumerReleaseFrame, cudaProducerReturnFrame, cudaDeviceSynchronize, cudaDeviceCreateProducer, cudaProducerDeinit, cudaProducerPrepareFrame, cudaGetValueMismatch, cudaConsumerAcquireFrame, cudaProducerInit, cudaDeviceCreateConsumer
|
||||
|
||||
## Dependencies needed to build/run
|
||||
[EGL](../../../README.md#egl)
|
||||
|
||||
## Prerequisites
|
||||
|
||||
Download and install the [CUDA Toolkit](https://developer.nvidia.com/cuda-downloads) for your corresponding platform.
|
||||
Make sure the dependencies mentioned in [Dependencies]() section above are installed.
|
||||
|
||||
## References (for more details)
|
||||
@@ -0,0 +1,265 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Simple CUDA consumer rendering sample app
|
||||
//
|
||||
|
||||
#include "cuda_consumer.h"
|
||||
|
||||
#include <cuda_runtime.h>
|
||||
#include <math.h>
|
||||
#include <unistd.h>
|
||||
|
||||
#include "eglstrm_common.h"
|
||||
|
||||
#if defined(EXTENSION_LIST)
|
||||
EXTENSION_LIST(EXTLST_EXTERN)
|
||||
#endif
|
||||
CUgraphicsResource cudaResource;
|
||||
|
||||
static int count_acq = 0;
|
||||
static double acquire_time[25000] = {0}, total_time_acq = 0;
|
||||
|
||||
static int count_rel = 0;
|
||||
static double rel_time[25000] = {0}, total_time_rel = 0;
|
||||
|
||||
void acquireApiStat(void);
|
||||
void acquireApiStat(void)
|
||||
{
|
||||
int i = 0;
|
||||
double min = 10000000, max = 0;
|
||||
double average_launch_time = 0, standard_deviation = 0;
|
||||
if (count_acq == 0)
|
||||
return;
|
||||
// lets compute the standard deviation
|
||||
min = max = acquire_time[1];
|
||||
average_launch_time = (total_time_acq - acquire_time[0]) / count_acq;
|
||||
for (i = 1; i < count_acq; i++) {
|
||||
standard_deviation += (acquire_time[i] - average_launch_time) * (acquire_time[i] - average_launch_time);
|
||||
if (acquire_time[i] < min)
|
||||
min = acquire_time[i];
|
||||
if (acquire_time[i] > max)
|
||||
max = acquire_time[i];
|
||||
}
|
||||
standard_deviation = sqrt(standard_deviation / count_acq);
|
||||
printf("acquire Avg: %lf\n", average_launch_time);
|
||||
printf("acquire SD: %lf\n", standard_deviation);
|
||||
printf("acquire min: %lf\n", min);
|
||||
printf("acquire max: %lf\n", max);
|
||||
|
||||
min = max = rel_time[1];
|
||||
average_launch_time = (total_time_rel - rel_time[0]) / count_rel;
|
||||
for (i = 1; i < count_rel; i++) {
|
||||
standard_deviation += (rel_time[i] - average_launch_time) * (rel_time[i] - average_launch_time);
|
||||
if (rel_time[i] < min)
|
||||
min = rel_time[i];
|
||||
if (rel_time[i] > max)
|
||||
max = rel_time[i];
|
||||
}
|
||||
standard_deviation = sqrt(standard_deviation / count_rel);
|
||||
printf("release Avg: %lf\n", average_launch_time);
|
||||
printf("release SD: %lf\n", standard_deviation);
|
||||
printf("release min: %lf\n", min);
|
||||
printf("release max: %lf\n", max);
|
||||
}
|
||||
CUresult cudaConsumerAcquireFrame(test_cuda_consumer_s *cudaConsumer, int frameNumber)
|
||||
{
|
||||
CUresult cuStatus = CUDA_SUCCESS;
|
||||
CUeglFrame cudaEgl;
|
||||
struct timespec start, end;
|
||||
EGLint streamState = 0;
|
||||
double curTime;
|
||||
|
||||
if (!cudaConsumer) {
|
||||
printf("%s: Bad parameter\n", __func__);
|
||||
goto done;
|
||||
}
|
||||
|
||||
while (1) {
|
||||
if (!eglQueryStreamKHR(cudaConsumer->eglDisplay, cudaConsumer->eglStream, EGL_STREAM_STATE_KHR, &streamState)) {
|
||||
printf("Cuda Consumer: eglQueryStreamKHR EGL_STREAM_STATE_KHR failed\n");
|
||||
cuStatus = CUDA_ERROR_UNKNOWN;
|
||||
goto done;
|
||||
}
|
||||
if (streamState == EGL_STREAM_STATE_DISCONNECTED_KHR) {
|
||||
printf("Cuda Consumer: EGL_STREAM_STATE_DISCONNECTED_KHR received\n");
|
||||
cuStatus = CUDA_ERROR_UNKNOWN;
|
||||
goto done;
|
||||
}
|
||||
|
||||
if (streamState == EGL_STREAM_STATE_NEW_FRAME_AVAILABLE_KHR) {
|
||||
break;
|
||||
}
|
||||
}
|
||||
if (cudaConsumer->profileAPI) {
|
||||
getTime(&start);
|
||||
}
|
||||
cuStatus =
|
||||
cuEGLStreamConsumerAcquireFrame(&(cudaConsumer->cudaConn), &cudaResource, &cudaConsumer->consCudaStream, 16000);
|
||||
if (cudaConsumer->profileAPI) {
|
||||
getTime(&end);
|
||||
curTime = TIME_DIFF(end, start);
|
||||
acquire_time[count_acq++] = curTime;
|
||||
if (count_acq == 25000)
|
||||
count_acq = 0;
|
||||
total_time_acq += curTime;
|
||||
}
|
||||
if (cuStatus == CUDA_SUCCESS) {
|
||||
CUdeviceptr pDevPtr = 0;
|
||||
cudaError_t err;
|
||||
|
||||
cuStatus = cuGraphicsResourceGetMappedEglFrame(&cudaEgl, cudaResource, 0, 0);
|
||||
if (cuStatus != CUDA_SUCCESS) {
|
||||
printf("Cuda get resource failed with %d\n", cuStatus);
|
||||
goto done;
|
||||
}
|
||||
pDevPtr = (CUdeviceptr)cudaEgl.frame.pPitch[0];
|
||||
|
||||
err = cudaConsumer_filter(cudaConsumer->consCudaStream,
|
||||
(char *)pDevPtr,
|
||||
WIDTH * 4,
|
||||
HEIGHT,
|
||||
PROD_DATA + frameNumber,
|
||||
CONS_DATA + frameNumber,
|
||||
frameNumber);
|
||||
if (err != cudaSuccess) {
|
||||
printf("Cuda Consumer: kernel failed with: %s\n", cudaGetErrorString(err));
|
||||
goto done;
|
||||
}
|
||||
}
|
||||
|
||||
done:
|
||||
return cuStatus;
|
||||
}
|
||||
|
||||
CUresult cudaConsumerReleaseFrame(test_cuda_consumer_s *cudaConsumer, int frameNumber)
|
||||
{
|
||||
CUresult cuStatus = CUDA_SUCCESS;
|
||||
struct timespec start, end;
|
||||
double curTime;
|
||||
|
||||
if (!cudaConsumer) {
|
||||
printf("%s: Bad parameter\n", __func__);
|
||||
goto done;
|
||||
}
|
||||
if (cudaConsumer->profileAPI) {
|
||||
getTime(&start);
|
||||
}
|
||||
cuStatus = cuEGLStreamConsumerReleaseFrame(&cudaConsumer->cudaConn, cudaResource, &cudaConsumer->consCudaStream);
|
||||
if (cudaConsumer->profileAPI) {
|
||||
getTime(&end);
|
||||
curTime = TIME_DIFF(end, start);
|
||||
rel_time[count_rel++] = curTime;
|
||||
if (count_rel == 25000)
|
||||
count_rel = 0;
|
||||
total_time_rel += curTime;
|
||||
}
|
||||
if (cuStatus != CUDA_SUCCESS) {
|
||||
printf("cuEGLStreamConsumerReleaseFrame failed, status:%d\n", cuStatus);
|
||||
goto done;
|
||||
}
|
||||
|
||||
done:
|
||||
return cuStatus;
|
||||
}
|
||||
|
||||
CUresult cudaDeviceCreateConsumer(test_cuda_consumer_s *cudaConsumer)
|
||||
{
|
||||
CUdevice device;
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
CUctxCreateParams ctxCreateParams = {};
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuInit(0))) {
|
||||
printf("Failed to initialize CUDA\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuDeviceGet(&device, cudaConsumer->cudaDevId))) {
|
||||
printf("failed to get CUDA device\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuCtxCreate(&cudaConsumer->context, &ctxCreateParams, 0, device))) {
|
||||
printf("failed to create CUDA context\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
int major = 0, minor = 0;
|
||||
char deviceName[256];
|
||||
cuDeviceGetAttribute(&major, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MAJOR, device);
|
||||
cuDeviceGetAttribute(&minor, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MINOR, device);
|
||||
cuDeviceGetName(deviceName, 256, device);
|
||||
printf("CUDA Consumer on GPU Device %d: \"%s\" with compute capability "
|
||||
"%d.%d\n\n",
|
||||
device,
|
||||
deviceName,
|
||||
major,
|
||||
minor);
|
||||
|
||||
cuCtxPopCurrent(&cudaConsumer->context);
|
||||
if (major < 6) {
|
||||
printf("EGLStream_CUDA_CrossGPU requires SM 6.0 or higher arch GPU. "
|
||||
"Exiting...\n");
|
||||
exit(2); // EXIT_WAIVED
|
||||
}
|
||||
|
||||
return status;
|
||||
}
|
||||
|
||||
CUresult cuda_consumer_init(test_cuda_consumer_s *cudaConsumer, TestArgs *args)
|
||||
{
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
int bufferSize;
|
||||
|
||||
cudaConsumer->charCnt = args->charCnt;
|
||||
bufferSize = args->charCnt;
|
||||
|
||||
cudaConsumer->pCudaCopyMem = (unsigned char *)malloc(bufferSize);
|
||||
if (cudaConsumer->pCudaCopyMem == NULL) {
|
||||
printf("Cuda Consumer: malloc failed\n");
|
||||
goto done;
|
||||
}
|
||||
|
||||
status = cuStreamCreate(&cudaConsumer->consCudaStream, 0);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer: cuStreamCreate failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
|
||||
atexit(acquireApiStat);
|
||||
done:
|
||||
return status;
|
||||
}
|
||||
|
||||
CUresult cuda_consumer_Deinit(test_cuda_consumer_s *cudaConsumer)
|
||||
{
|
||||
if (cudaConsumer->pCudaCopyMem) {
|
||||
free(cudaConsumer->pCudaCopyMem);
|
||||
}
|
||||
return cuEGLStreamConsumerDisconnect(&cudaConsumer->cudaConn);
|
||||
}
|
||||
@@ -0,0 +1,72 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: CUDA consumer header file
|
||||
//
|
||||
|
||||
#ifndef _CUDA_CONSUMER_H_
|
||||
#define _CUDA_CONSUMER_H_
|
||||
|
||||
#include <cuda.h>
|
||||
#include <cuda_runtime.h>
|
||||
#include <stdio.h>
|
||||
#include <stdlib.h>
|
||||
#include <string.h>
|
||||
|
||||
#include "cudaEGL.h"
|
||||
#include "eglstrm_common.h"
|
||||
|
||||
typedef struct _test_cuda_consumer_s
|
||||
{
|
||||
CUcontext context;
|
||||
CUeglStreamConnection cudaConn;
|
||||
int cudaDevId;
|
||||
EGLDisplay eglDisplay;
|
||||
EGLStreamKHR eglStream;
|
||||
unsigned int charCnt;
|
||||
char *cudaBuf;
|
||||
bool profileAPI;
|
||||
unsigned char *pCudaCopyMem;
|
||||
CUstream consCudaStream;
|
||||
} test_cuda_consumer_s;
|
||||
|
||||
CUresult cuda_consumer_init(test_cuda_consumer_s *cudaConsumer, TestArgs *args);
|
||||
CUresult cuda_consumer_Deinit(test_cuda_consumer_s *cudaConsumer);
|
||||
CUresult cudaConsumerAcquireFrame(test_cuda_consumer_s *data, int frameNumber);
|
||||
CUresult cudaConsumerReleaseFrame(test_cuda_consumer_s *data, int frameNumber);
|
||||
CUresult cudaDeviceCreateConsumer(test_cuda_consumer_s *cudaConsumer);
|
||||
cudaError_t cudaConsumer_filter(CUstream cStream,
|
||||
char *pSrc,
|
||||
int width,
|
||||
int height,
|
||||
char expectedVal,
|
||||
char newVal,
|
||||
int frameNumber);
|
||||
cudaError_t cudaGetValueMismatch(void);
|
||||
|
||||
#endif
|
||||
@@ -0,0 +1,294 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Simple cuda EGL stream producer app
|
||||
//
|
||||
|
||||
#include "cuda_producer.h"
|
||||
|
||||
#include <cuda_runtime.h>
|
||||
|
||||
#include "cudaEGL.h"
|
||||
#include "eglstrm_common.h"
|
||||
#if defined(EXTENSION_LIST)
|
||||
EXTENSION_LIST(EXTLST_EXTERN)
|
||||
#endif
|
||||
#include <stdio.h>
|
||||
#include <stdlib.h>
|
||||
#include <string.h>
|
||||
|
||||
#include "cuda_runtime.h"
|
||||
#include "math.h"
|
||||
|
||||
int cudaPresentReturnData = INIT_DATA;
|
||||
int fakePresent = 0;
|
||||
CUeglFrame fakeFrame;
|
||||
CUdeviceptr cudaPtrFake;
|
||||
extern bool isCrossDevice;
|
||||
|
||||
void cudaProducerPrepareFrame(CUeglFrame *cudaEgl, CUdeviceptr cudaPtr, int bufferSize)
|
||||
{
|
||||
cudaEgl->frame.pPitch[0] = (void *)cudaPtr;
|
||||
cudaEgl->width = WIDTH;
|
||||
cudaEgl->depth = 0;
|
||||
cudaEgl->height = HEIGHT;
|
||||
cudaEgl->pitch = WIDTH * 4;
|
||||
cudaEgl->frameType = CU_EGL_FRAME_TYPE_PITCH;
|
||||
cudaEgl->planeCount = 1;
|
||||
cudaEgl->numChannels = 4;
|
||||
cudaEgl->eglColorFormat = CU_EGL_COLOR_FORMAT_ARGB;
|
||||
cudaEgl->cuFormat = CU_AD_FORMAT_UNSIGNED_INT8;
|
||||
}
|
||||
|
||||
static int count_present = 0, count_return = 0;
|
||||
static double present_time[25000] = {0}, total_time_present = 0;
|
||||
static double return_time[25000] = {0}, total_time_return = 0;
|
||||
|
||||
void presentApiStat(void);
|
||||
void presentApiStat(void)
|
||||
{
|
||||
int i = 0;
|
||||
double min = 10000000, max = 0;
|
||||
double average_launch_time = 0, standard_deviation = 0;
|
||||
if (count_present == 0)
|
||||
return;
|
||||
// lets compute the standard deviation
|
||||
min = max = present_time[1];
|
||||
average_launch_time = (total_time_present) / count_present;
|
||||
for (i = 1; i < count_present; i++) {
|
||||
standard_deviation += (present_time[i] - average_launch_time) * (present_time[i] - average_launch_time);
|
||||
if (present_time[i] < min)
|
||||
min = present_time[i];
|
||||
if (present_time[i] > max)
|
||||
max = present_time[i];
|
||||
}
|
||||
standard_deviation = sqrt(standard_deviation / count_present);
|
||||
printf("present Avg: %lf\n", average_launch_time);
|
||||
printf("present SD: %lf\n", standard_deviation);
|
||||
printf("present min: %lf\n", min);
|
||||
printf("present max: %lf\n", max);
|
||||
|
||||
min = max = return_time[1];
|
||||
average_launch_time = (total_time_return - return_time[0]) / count_return;
|
||||
for (i = 1; i < count_return; i++) {
|
||||
standard_deviation += (return_time[i] - average_launch_time) * (return_time[i] - average_launch_time);
|
||||
if (return_time[i] < min)
|
||||
min = return_time[i];
|
||||
if (return_time[i] > max)
|
||||
max = return_time[i];
|
||||
}
|
||||
standard_deviation = sqrt(standard_deviation / count_return);
|
||||
printf("return Avg: %lf\n", average_launch_time);
|
||||
printf("return SD: %lf\n", standard_deviation);
|
||||
printf("return min: %lf\n", min);
|
||||
printf("return max: %lf\n", max);
|
||||
}
|
||||
CUresult cudaProducerPresentFrame(test_cuda_producer_s *cudaProducer, CUeglFrame cudaEgl, int t)
|
||||
{
|
||||
static int flag = 0;
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
struct timespec start, end;
|
||||
double curTime;
|
||||
CUdeviceptr pDevPtr = (CUdeviceptr)cudaEgl.frame.pPitch[0];
|
||||
cudaProducer_filter(
|
||||
cudaProducer->prodCudaStream, (char *)pDevPtr, WIDTH * 4, HEIGHT, cudaPresentReturnData, PROD_DATA + t, t);
|
||||
if (cudaProducer->profileAPI) {
|
||||
getTime(&start);
|
||||
}
|
||||
status = cuEGLStreamProducerPresentFrame(&cudaProducer->cudaConn, cudaEgl, &cudaProducer->prodCudaStream);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: Present frame failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
flag++;
|
||||
if (cudaProducer->profileAPI && flag > 10) {
|
||||
getTime(&end);
|
||||
curTime = TIME_DIFF(end, start);
|
||||
present_time[count_present++] = curTime;
|
||||
if (count_present == 25000)
|
||||
count_present = 0;
|
||||
total_time_present += curTime;
|
||||
}
|
||||
done:
|
||||
return status;
|
||||
}
|
||||
|
||||
int flag = 0;
|
||||
CUresult cudaProducerReturnFrame(test_cuda_producer_s *cudaProducer, CUeglFrame cudaEgl, int t)
|
||||
{
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
struct timespec start, end;
|
||||
double curTime;
|
||||
CUdeviceptr pDevPtr = 0;
|
||||
|
||||
pDevPtr = (CUdeviceptr)cudaEgl.frame.pPitch[0];
|
||||
if (cudaProducer->profileAPI) {
|
||||
getTime(&start);
|
||||
}
|
||||
|
||||
while (1) {
|
||||
status = cuEGLStreamProducerReturnFrame(&cudaProducer->cudaConn, &cudaEgl, &cudaProducer->prodCudaStream);
|
||||
if (status == CUDA_ERROR_LAUNCH_TIMEOUT) {
|
||||
continue;
|
||||
}
|
||||
else if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: Return frame failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
break;
|
||||
}
|
||||
if (cudaProducer->profileAPI) {
|
||||
getTime(&end);
|
||||
curTime = TIME_DIFF(end, start);
|
||||
return_time[count_return++] = curTime;
|
||||
if (count_return == 25000)
|
||||
count_return = 0;
|
||||
total_time_return += curTime;
|
||||
}
|
||||
if (flag % 2 == 0) {
|
||||
cudaPresentReturnData++;
|
||||
}
|
||||
cudaProducer_filter(
|
||||
cudaProducer->prodCudaStream, (char *)pDevPtr, WIDTH * 4, HEIGHT, CONS_DATA + t, cudaPresentReturnData, t);
|
||||
flag++;
|
||||
done:
|
||||
return status;
|
||||
}
|
||||
|
||||
CUresult cudaDeviceCreateProducer(test_cuda_producer_s *cudaProducer)
|
||||
{
|
||||
CUdevice device;
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
CUctxCreateParams ctxCreateParams = {};
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuInit(0))) {
|
||||
printf("Failed to initialize CUDA\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuDeviceGet(&device, cudaProducer->cudaDevId))) {
|
||||
printf("failed to get CUDA device\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS != (status = cuCtxCreate(&cudaProducer->context, &ctxCreateParams, 0, device))) {
|
||||
printf("failed to create CUDA context\n");
|
||||
return status;
|
||||
}
|
||||
|
||||
int major = 0, minor = 0;
|
||||
char deviceName[256];
|
||||
cuDeviceGetAttribute(&major, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MAJOR, device);
|
||||
cuDeviceGetAttribute(&minor, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MINOR, device);
|
||||
cuDeviceGetName(deviceName, 256, device);
|
||||
printf("CUDA Producer on GPU Device %d: \"%s\" with compute capability "
|
||||
"%d.%d\n\n",
|
||||
device,
|
||||
deviceName,
|
||||
major,
|
||||
minor);
|
||||
|
||||
cuCtxPopCurrent(&cudaProducer->context);
|
||||
|
||||
if (major < 6) {
|
||||
printf("EGLStream_CUDA_CrossGPU requires SM 6.0 or higher arch GPU. "
|
||||
"Exiting...\n");
|
||||
exit(2); // EXIT_WAIVED
|
||||
}
|
||||
|
||||
return status;
|
||||
}
|
||||
|
||||
CUresult cudaProducerInit(test_cuda_producer_s *cudaProducer, TestArgs *args)
|
||||
{
|
||||
CUresult status = CUDA_SUCCESS;
|
||||
int bufferSize;
|
||||
|
||||
cudaProducer->charCnt = args->charCnt;
|
||||
bufferSize = cudaProducer->charCnt;
|
||||
|
||||
cudaProducer->tempBuff = (char *)malloc(bufferSize);
|
||||
if (!cudaProducer->tempBuff) {
|
||||
printf("Cuda Producer: Failed to allocate image buffer\n");
|
||||
status = CUDA_ERROR_UNKNOWN;
|
||||
goto done;
|
||||
}
|
||||
memset((void *)cudaProducer->tempBuff, INIT_DATA, cudaProducer->charCnt);
|
||||
|
||||
// Fill this init data
|
||||
status = cuMemAlloc(&cudaProducer->cudaPtr, bufferSize);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuda Malloc failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
status = cuMemcpyHtoD(cudaProducer->cudaPtr, (void *)(cudaProducer->tempBuff), bufferSize);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuMemCpy failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
|
||||
// Fill this init data
|
||||
status = cuMemAlloc(&cudaProducer->cudaPtr1, bufferSize);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuda Malloc failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
status = cuMemcpyHtoD(cudaProducer->cudaPtr1, (void *)(cudaProducer->tempBuff), bufferSize);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuMemCpy failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
|
||||
status = cuStreamCreate(&cudaProducer->prodCudaStream, 0);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuStreamCreate failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
|
||||
// Fill this init data
|
||||
status = cuMemAlloc(&cudaPtrFake, 100);
|
||||
if (status != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: cuda Malloc failed, status:%d\n", status);
|
||||
goto done;
|
||||
}
|
||||
|
||||
atexit(presentApiStat);
|
||||
done:
|
||||
return status;
|
||||
}
|
||||
|
||||
CUresult cudaProducerDeinit(test_cuda_producer_s *cudaProducer)
|
||||
{
|
||||
if (cudaProducer->tempBuff) {
|
||||
free(cudaProducer->tempBuff);
|
||||
}
|
||||
if (cudaProducer->cudaPtr) {
|
||||
cuMemFree(cudaProducer->cudaPtr);
|
||||
}
|
||||
return cuEGLStreamProducerDisconnect(&cudaProducer->cudaConn);
|
||||
}
|
||||
@@ -0,0 +1,71 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Simple cuda producer header file
|
||||
//
|
||||
|
||||
#ifndef _CUDA_PRODUCER_H_
|
||||
#define _CUDA_PRODUCER_H_
|
||||
#include <EGL/egl.h>
|
||||
#include <EGL/eglext.h>
|
||||
#include <cuda.h>
|
||||
#include <cuda_runtime.h>
|
||||
|
||||
#include "cudaEGL.h"
|
||||
#include "eglstrm_common.h"
|
||||
|
||||
typedef struct _test_cuda_producer_s
|
||||
{
|
||||
// Stream params
|
||||
CUcontext context;
|
||||
CUeglStreamConnection cudaConn;
|
||||
int cudaDevId;
|
||||
EGLStreamKHR eglStream;
|
||||
EGLDisplay eglDisplay;
|
||||
unsigned int charCnt;
|
||||
bool profileAPI;
|
||||
char *tempBuff;
|
||||
CUdeviceptr cudaPtr;
|
||||
CUdeviceptr cudaPtr1;
|
||||
CUstream prodCudaStream;
|
||||
} test_cuda_producer_s;
|
||||
|
||||
CUresult cudaProducerInit(test_cuda_producer_s *cudaProducer, TestArgs *args);
|
||||
CUresult cudaProducerPresentFrame(test_cuda_producer_s *parserArg, CUeglFrame cudaEgl, int t);
|
||||
CUresult cudaProducerReturnFrame(test_cuda_producer_s *parserArg, CUeglFrame cudaEgl, int t);
|
||||
CUresult cudaProducerDeinit(test_cuda_producer_s *cudaProducer);
|
||||
CUresult cudaDeviceCreateProducer(test_cuda_producer_s *cudaProducer);
|
||||
cudaError_t cudaProducer_filter(CUstream cStream,
|
||||
char *pSrc,
|
||||
int width,
|
||||
int height,
|
||||
char expectedVal,
|
||||
char newVal,
|
||||
int frameNumber);
|
||||
void cudaProducerPrepareFrame(CUeglFrame *cudaEgl, CUdeviceptr cudaPtr, int bufferSize);
|
||||
#endif
|
||||
@@ -0,0 +1,424 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Common egl stream functions
|
||||
//
|
||||
|
||||
#include "eglstrm_common.h"
|
||||
|
||||
EGLStreamKHR g_producerEglStream = EGL_NO_STREAM_KHR;
|
||||
EGLStreamKHR g_consumerEglStream = EGL_NO_STREAM_KHR;
|
||||
EGLDisplay g_producerEglDisplay = EGL_NO_DISPLAY;
|
||||
EGLDisplay g_consumerEglDisplay = EGL_NO_DISPLAY;
|
||||
int cudaDevIndexProd = -1;
|
||||
int cudaDevIndexCons = -1;
|
||||
|
||||
#if defined(EXTENSION_LIST)
|
||||
EXTENSION_LIST(EXTLST_DECL)
|
||||
typedef void (*extlst_fnptr_t)(void);
|
||||
static struct
|
||||
{
|
||||
extlst_fnptr_t *fnptr;
|
||||
char const *name;
|
||||
bool is_dgpu; // This function is need only for dgpu case
|
||||
} extensionList[] = {EXTENSION_LIST(EXTLST_ENTRY)};
|
||||
|
||||
int eglSetupExtensions(bool isCrossDevice)
|
||||
{
|
||||
unsigned int i;
|
||||
|
||||
for (i = 0; i < (sizeof(extensionList) / sizeof(*extensionList)); i++) {
|
||||
// load the dgpu function only if we are running cross device test
|
||||
if ((!extensionList[i].is_dgpu) || (extensionList[i].is_dgpu == isCrossDevice)) {
|
||||
*extensionList[i].fnptr = eglGetProcAddress(extensionList[i].name);
|
||||
if (*extensionList[i].fnptr == NULL) {
|
||||
printf("Couldn't get address of %s()\n", extensionList[i].name);
|
||||
return 0;
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
return 1;
|
||||
}
|
||||
|
||||
int EGLStreamInit(bool isCrossDevice, int isConsumer, EGLNativeFileDescriptorKHR fileDesc)
|
||||
{
|
||||
static const EGLint streamAttrFIFOMode[] = {
|
||||
EGL_STREAM_FIFO_LENGTH_KHR, 5, EGL_SUPPORT_REUSE_NV, EGL_FALSE, EGL_NONE};
|
||||
EGLDisplay eglDisplay[2] = {0};
|
||||
EGLStreamKHR eglStream[2] = {0};
|
||||
EGLBoolean eglStatus;
|
||||
|
||||
#define MAX_EGL_DEVICES 4
|
||||
|
||||
EGLDeviceEXT devices[MAX_EGL_DEVICES];
|
||||
EGLint numDevices = 0;
|
||||
|
||||
eglStatus = eglQueryDevicesEXT(MAX_EGL_DEVICES, devices, &numDevices);
|
||||
if (eglStatus != EGL_TRUE) {
|
||||
printf("Error querying EGL devices\n");
|
||||
goto Done;
|
||||
}
|
||||
|
||||
if (numDevices == 0) {
|
||||
printf("No EGL devices found\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
// If cross device, create discrete GPU stream first and then create the
|
||||
// integrated GPU stream to connect to it via fd. The other way round fails
|
||||
// in producer connect.
|
||||
//
|
||||
// TODO: Find out if this EGL behavior is by design.
|
||||
if (isConsumer) {
|
||||
int egl_device_id = 0;
|
||||
for (egl_device_id = 0; egl_device_id < numDevices; egl_device_id++) {
|
||||
EGLAttrib cuda_device;
|
||||
eglStatus = eglQueryDeviceAttribEXT(devices[egl_device_id], EGL_CUDA_DEVICE_NV, &cuda_device);
|
||||
if (eglStatus == EGL_TRUE) {
|
||||
cudaDevIndexCons = cuda_device; // We select first EGL-CUDA Capable
|
||||
// device for consumer.
|
||||
printf("Found EGL-CUDA Capable device with CUDA Device id = %d out of "
|
||||
"egl_device_id = %d\n",
|
||||
(int)cudaDevIndexCons,
|
||||
egl_device_id);
|
||||
break;
|
||||
}
|
||||
}
|
||||
|
||||
if (egl_device_id >= numDevices) {
|
||||
printf("No CUDA Capable EGL Device found.. Waiving execution\n");
|
||||
goto Done;
|
||||
}
|
||||
|
||||
g_consumerEglDisplay = eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT, (void *)devices[egl_device_id], NULL);
|
||||
if (g_consumerEglDisplay == EGL_NO_DISPLAY) {
|
||||
printf("Could not get EGL display from device. \n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
eglStatus = eglInitialize(g_consumerEglDisplay, 0, 0);
|
||||
if (!eglStatus) {
|
||||
printf("EGL failed to initialize. \n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
g_consumerEglStream = eglCreateStreamKHR(g_consumerEglDisplay, streamAttrFIFOMode);
|
||||
if (g_consumerEglStream == EGL_NO_STREAM_KHR) {
|
||||
printf("Could not create EGL stream.\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
eglStatus = eglStreamAttribKHR(g_consumerEglDisplay, g_consumerEglStream, EGL_CONSUMER_LATENCY_USEC_KHR, 16000);
|
||||
if (eglStatus != EGL_TRUE) {
|
||||
printf("eglStreamAttribKHR EGL_CONSUMER_LATENCY_USEC_KHR failed\n");
|
||||
goto Done;
|
||||
}
|
||||
|
||||
eglStatus =
|
||||
eglStreamAttribKHR(g_consumerEglDisplay, g_consumerEglStream, EGL_CONSUMER_ACQUIRE_TIMEOUT_USEC_KHR, 16000);
|
||||
if (eglStatus != EGL_TRUE) {
|
||||
printf("eglStreamAttribKHR EGL_CONSUMER_ACQUIRE_TIMEOUT_USEC_KHR failed\n");
|
||||
goto Done;
|
||||
}
|
||||
}
|
||||
|
||||
if (!isConsumer) { // Producer
|
||||
|
||||
if (fileDesc == EGL_NO_FILE_DESCRIPTOR_KHR) {
|
||||
printf("Cuda Producer received bad file descriptor\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
int egl_device_id = 0;
|
||||
int egl_cuda_devices = 0;
|
||||
for (egl_device_id = 0; egl_device_id < numDevices; egl_device_id++) {
|
||||
EGLAttrib cuda_device = -1;
|
||||
eglStatus = eglQueryDeviceAttribEXT(devices[egl_device_id], EGL_CUDA_DEVICE_NV, &cuda_device);
|
||||
if (eglStatus == EGL_TRUE) {
|
||||
egl_cuda_devices++;
|
||||
if (isCrossDevice && (egl_cuda_devices > 1)) {
|
||||
// We select second EGL-CUDA Capable device for producer.
|
||||
cudaDevIndexProd = (int)cuda_device;
|
||||
printf("Found EGL-CUDA Capable device with CUDA Device id = %d "
|
||||
"egl_device_id = %d \n",
|
||||
(int)cudaDevIndexProd,
|
||||
egl_device_id);
|
||||
break;
|
||||
}
|
||||
if (!isCrossDevice) {
|
||||
// We select first EGL-CUDA Capable device for producer same as
|
||||
// consumer.
|
||||
cudaDevIndexProd = (int)cuda_device;
|
||||
printf("Found EGL-CUDA Capable device with CUDA Device id = %d "
|
||||
"egl_device_id = %d \n",
|
||||
(int)cudaDevIndexProd,
|
||||
egl_device_id);
|
||||
break;
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
if (egl_device_id >= numDevices) {
|
||||
printf("No CUDA Capable EGL Device found.. Waiving execution\n");
|
||||
goto Done;
|
||||
}
|
||||
|
||||
g_producerEglDisplay = eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT, (void *)devices[egl_device_id], NULL);
|
||||
if (g_producerEglDisplay == EGL_NO_DISPLAY) {
|
||||
printf("Could not get EGL display from device. \n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
eglStatus = eglInitialize(g_producerEglDisplay, 0, 0);
|
||||
if (!eglStatus) {
|
||||
printf("EGL failed to initialize. \n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
|
||||
g_producerEglStream = eglCreateStreamFromFileDescriptorKHR(g_producerEglDisplay, fileDesc);
|
||||
close(fileDesc);
|
||||
|
||||
if (g_producerEglStream == EGL_NO_STREAM_KHR) {
|
||||
printf("CUDA Producer Could not create EGL stream.\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
goto Done;
|
||||
}
|
||||
else {
|
||||
printf("Producer created EGLStream for the GPU.\n");
|
||||
}
|
||||
}
|
||||
|
||||
Done:
|
||||
return eglStatus == EGL_TRUE ? 1 : 0;
|
||||
}
|
||||
|
||||
void EGLStreamFini(void)
|
||||
{
|
||||
if (g_producerEglStream != EGL_NO_STREAM_KHR) {
|
||||
eglDestroyStreamKHR(g_producerEglDisplay, g_producerEglStream);
|
||||
}
|
||||
if (g_consumerEglStream != g_producerEglStream) {
|
||||
if (g_consumerEglStream != EGL_NO_STREAM_KHR) {
|
||||
eglDestroyStreamKHR(g_consumerEglDisplay, g_consumerEglStream);
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
int UnixSocketConnect(const char *socket_name)
|
||||
{
|
||||
int sock_fd = -1;
|
||||
struct sockaddr_un sock_addr;
|
||||
int wait_loop = 0;
|
||||
|
||||
sock_fd = socket(PF_UNIX, SOCK_STREAM, 0);
|
||||
if (sock_fd < 0) {
|
||||
printf("%s: socket create failed.\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
if (verbose)
|
||||
printf("%s: send_fd: sock_fd: %d\n", __func__, sock_fd);
|
||||
|
||||
memset(&sock_addr, 0, sizeof(struct sockaddr_un));
|
||||
sock_addr.sun_family = AF_UNIX;
|
||||
strncpy(sock_addr.sun_path, socket_name, sizeof(sock_addr.sun_path) - 1);
|
||||
|
||||
while (connect(sock_fd, (const struct sockaddr *)&sock_addr, sizeof(struct sockaddr_un))) {
|
||||
if (wait_loop < 60) {
|
||||
if (!wait_loop)
|
||||
printf("Waiting for EGL stream producer ");
|
||||
else
|
||||
printf(".");
|
||||
fflush(stdout);
|
||||
sleep(1);
|
||||
wait_loop++;
|
||||
}
|
||||
else {
|
||||
printf("\n%s: Waiting timed out\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
}
|
||||
if (wait_loop)
|
||||
printf("\n");
|
||||
|
||||
if (verbose)
|
||||
printf("%s: Wait is done\n", __func__);
|
||||
|
||||
return sock_fd;
|
||||
}
|
||||
|
||||
/* Send <fd_to_send> (a file descriptor) to another process */
|
||||
/* over a unix domain socket named <socket_name>. */
|
||||
/* <socket_name> can be any nonexistant filename. */
|
||||
int EGLStreamSendfd(int send_fd, int fd_to_send)
|
||||
{
|
||||
struct msghdr msg;
|
||||
struct iovec iov[1];
|
||||
char ctrl_buf[CMSG_SPACE(sizeof(int))];
|
||||
struct cmsghdr *cmsg = NULL;
|
||||
void *data;
|
||||
int res;
|
||||
memset(&msg, 0, sizeof(msg));
|
||||
|
||||
iov[0].iov_len = 1; // must send at least 1 byte
|
||||
iov[0].iov_base = (void *)"x"; // any byte value (value ignored)
|
||||
msg.msg_iov = iov;
|
||||
msg.msg_iovlen = 1;
|
||||
|
||||
memset(ctrl_buf, 0, sizeof(ctrl_buf));
|
||||
msg.msg_control = ctrl_buf;
|
||||
msg.msg_controllen = sizeof(ctrl_buf);
|
||||
|
||||
cmsg = CMSG_FIRSTHDR(&msg);
|
||||
cmsg->cmsg_level = SOL_SOCKET;
|
||||
cmsg->cmsg_type = SCM_RIGHTS;
|
||||
cmsg->cmsg_len = CMSG_LEN(sizeof(int));
|
||||
data = CMSG_DATA(cmsg);
|
||||
*(int *)data = fd_to_send;
|
||||
|
||||
msg.msg_controllen = cmsg->cmsg_len;
|
||||
|
||||
res = sendmsg(send_fd, &msg, 0);
|
||||
if (res <= 0) {
|
||||
printf("%s: sendmsg failed", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
return 0;
|
||||
}
|
||||
|
||||
/* Listen on a unix domain socket named <socket_name>. */
|
||||
/* Connect to it and return connect_fd */
|
||||
int UnixSocketCreate(const char *socket_name)
|
||||
{
|
||||
int listen_fd;
|
||||
struct sockaddr_un sock_addr;
|
||||
int connect_fd;
|
||||
struct sockaddr_un connect_addr;
|
||||
socklen_t connect_addr_len = 0;
|
||||
|
||||
listen_fd = socket(PF_UNIX, SOCK_STREAM, 0);
|
||||
if (listen_fd < 0) {
|
||||
printf("%s: socket create failed", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
if (verbose)
|
||||
printf("%s: listen_fd: %d\n", __func__, listen_fd);
|
||||
|
||||
unlink(socket_name);
|
||||
|
||||
memset(&sock_addr, 0, sizeof(struct sockaddr_un));
|
||||
sock_addr.sun_family = AF_UNIX;
|
||||
strncpy(sock_addr.sun_path, socket_name, sizeof(sock_addr.sun_path) - 1);
|
||||
|
||||
if (bind(listen_fd, (const struct sockaddr *)&sock_addr, sizeof(struct sockaddr_un))) {
|
||||
printf("i%s: bind error", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
if (listen(listen_fd, 1)) {
|
||||
printf("%s: listen error", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
connect_fd = accept(listen_fd, (struct sockaddr *)&connect_addr, &connect_addr_len);
|
||||
|
||||
if (verbose)
|
||||
printf("%s: connect_fd: %d\n", __func__, connect_fd);
|
||||
|
||||
close(listen_fd);
|
||||
unlink(socket_name);
|
||||
if (connect_fd < 0) {
|
||||
printf("%s: accept failed\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
return connect_fd;
|
||||
}
|
||||
|
||||
/* receive a file descriptor from another process. */
|
||||
/* Returns the file descriptor. Note: the integer value */
|
||||
/* of the file descriptor may be different from the */
|
||||
/* integer value in the other process, but the file */
|
||||
/* descriptors in each process will refer to the same file */
|
||||
/* object in the kernel. */
|
||||
int EGLStreamReceivefd(int connect_fd)
|
||||
{
|
||||
struct msghdr msg;
|
||||
struct iovec iov[1];
|
||||
char msg_buf[1];
|
||||
char ctrl_buf[CMSG_SPACE(sizeof(int))];
|
||||
struct cmsghdr *cmsg;
|
||||
void *data;
|
||||
int recvfd;
|
||||
|
||||
memset(&msg, 0, sizeof(msg));
|
||||
|
||||
iov[0].iov_base = msg_buf;
|
||||
iov[0].iov_len = sizeof(msg_buf);
|
||||
msg.msg_iov = iov;
|
||||
msg.msg_iovlen = 1;
|
||||
|
||||
msg.msg_control = ctrl_buf;
|
||||
msg.msg_controllen = sizeof(ctrl_buf);
|
||||
|
||||
if (recvmsg(connect_fd, &msg, 0) <= 0) {
|
||||
printf("%s: recvmsg failed", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
cmsg = CMSG_FIRSTHDR(&msg);
|
||||
if (!cmsg) {
|
||||
printf("%s: NULL message header\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
if (cmsg->cmsg_level != SOL_SOCKET) {
|
||||
printf("%s: Message level is not SOL_SOCKET\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
if (cmsg->cmsg_type != SCM_RIGHTS) {
|
||||
printf("%s: Message type is not SCM_RIGHTS\n", __func__);
|
||||
return -1;
|
||||
}
|
||||
|
||||
data = CMSG_DATA(cmsg);
|
||||
recvfd = *(int *)data;
|
||||
|
||||
return recvfd;
|
||||
}
|
||||
|
||||
#endif
|
||||
@@ -0,0 +1,109 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Common EGL stream functions header file
|
||||
//
|
||||
|
||||
#ifndef _EGLSTRM_COMMON_H_
|
||||
#define _EGLSTRM_COMMON_H_
|
||||
|
||||
#include <signal.h>
|
||||
#include <stdbool.h>
|
||||
#include <stdio.h>
|
||||
#include <stdlib.h>
|
||||
#include <string.h>
|
||||
#include <sys/socket.h>
|
||||
#include <sys/stat.h>
|
||||
#include <sys/time.h>
|
||||
#include <sys/types.h>
|
||||
#include <sys/un.h>
|
||||
#include <time.h>
|
||||
#include <unistd.h>
|
||||
|
||||
#include "cuda.h"
|
||||
#include "cudaEGL.h"
|
||||
#define TIME_DIFF(end, start) (getMicrosecond(end) - getMicrosecond(start))
|
||||
|
||||
extern EGLStreamKHR g_producerEglStream;
|
||||
extern EGLStreamKHR g_consumerEglStream;
|
||||
extern EGLDisplay g_producerEglDisplay;
|
||||
extern EGLDisplay g_consumerEglDisplay;
|
||||
extern int cudaDevIndexCons;
|
||||
extern int cudaDevIndexProd;
|
||||
extern bool verbose;
|
||||
|
||||
#define EXTENSION_LIST(T) \
|
||||
T(PFNEGLCREATESTREAMKHRPROC, eglCreateStreamKHR) \
|
||||
T(PFNEGLDESTROYSTREAMKHRPROC, eglDestroyStreamKHR) \
|
||||
T(PFNEGLQUERYSTREAMKHRPROC, eglQueryStreamKHR) \
|
||||
T(PFNEGLQUERYSTREAMU64KHRPROC, eglQueryStreamu64KHR) \
|
||||
T(PFNEGLQUERYSTREAMTIMEKHRPROC, eglQueryStreamTimeKHR) \
|
||||
T(PFNEGLSTREAMATTRIBKHRPROC, eglStreamAttribKHR) \
|
||||
T(PFNEGLSTREAMCONSUMERACQUIREKHRPROC, eglStreamConsumerAcquireKHR) \
|
||||
T(PFNEGLSTREAMCONSUMERRELEASEKHRPROC, eglStreamConsumerReleaseKHR) \
|
||||
T(PFNEGLSTREAMCONSUMERGLTEXTUREEXTERNALKHRPROC, eglStreamConsumerGLTextureExternalKHR) \
|
||||
T(PFNEGLQUERYDEVICESEXTPROC, eglQueryDevicesEXT) \
|
||||
T(PFNEGLGETPLATFORMDISPLAYEXTPROC, eglGetPlatformDisplayEXT) \
|
||||
T(PFNEGLGETSTREAMFILEDESCRIPTORKHRPROC, eglGetStreamFileDescriptorKHR) \
|
||||
T(PFNEGLQUERYDEVICEATTRIBEXTPROC, eglQueryDeviceAttribEXT) \
|
||||
T(PFNEGLCREATESTREAMFROMFILEDESCRIPTORKHRPROC, eglCreateStreamFromFileDescriptorKHR)
|
||||
|
||||
#define EXTLST_DECL(tx, x) tx x = NULL;
|
||||
#define EXTLST_EXTERN(tx, x) extern tx x;
|
||||
#define EXTLST_ENTRY(tx, x) {(extlst_fnptr_t *)&x, #x},
|
||||
|
||||
#define MAX_STRING_SIZE 256
|
||||
#define INIT_DATA 0x01
|
||||
#define PROD_DATA 0x07
|
||||
#define CONS_DATA 0x04
|
||||
|
||||
#define SOCK_PATH "/tmp/tegra_sw_egl_socket"
|
||||
|
||||
typedef struct _TestArgs
|
||||
{
|
||||
unsigned int charCnt;
|
||||
bool isProducer;
|
||||
} TestArgs;
|
||||
|
||||
extern int WIDTH, HEIGHT;
|
||||
|
||||
int eglSetupExtensions(bool is_dgpu);
|
||||
int EGLStreamInit(bool isCrossDevice, int isConsumer, EGLNativeFileDescriptorKHR fileDesc);
|
||||
void EGLStreamFini(void);
|
||||
|
||||
int EGLStreamSetAttr(EGLDisplay display, EGLStreamKHR eglStream);
|
||||
int UnixSocketConnect(const char *socket_name);
|
||||
int EGLStreamSendfd(int send_fd, int fd_to_send);
|
||||
int UnixSocketCreate(const char *socket_name);
|
||||
int EGLStreamReceivefd(int connect_fd);
|
||||
|
||||
static clockid_t clock_id = CLOCK_MONOTONIC; // CLOCK_PROCESS_CPUTIME_ID;
|
||||
static double getMicrosecond(struct timespec t) { return ((t.tv_sec) * 1000000.0 + (t.tv_nsec) / 1.0e3); }
|
||||
|
||||
static inline void getTime(struct timespec *t) { clock_gettime(clock_id, t); }
|
||||
#endif
|
||||
228
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/helper.h
Normal file
228
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/helper.h
Normal file
@@ -0,0 +1,228 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
#include "eglstrm_common.h"
|
||||
#if defined(EXTENSION_LIST)
|
||||
EXTENSION_LIST(EXTLST_EXTERN)
|
||||
#endif
|
||||
#include <cuda.h>
|
||||
|
||||
int parseCmdLine(int argc, char *argv[], TestArgs *args);
|
||||
void printUsage(void);
|
||||
int NUMTRIALS = 10;
|
||||
int profileAPIs = 0;
|
||||
|
||||
bool verbose = 0;
|
||||
bool isCrossDevice = 0;
|
||||
|
||||
// Parse the command line options. Returns FAILURE on a parse error, SUCCESS
|
||||
// otherwise.
|
||||
int parseCmdLine(int argc, char *argv[], TestArgs *args)
|
||||
{
|
||||
int i;
|
||||
|
||||
for (i = 1; i < argc; i++) {
|
||||
if (strcmp(argv[i], "-h") == 0) {
|
||||
printUsage();
|
||||
exit(0);
|
||||
}
|
||||
else if (strcmp(argv[i], "-n") == 0) {
|
||||
++i;
|
||||
if (sscanf(argv[i], "%d", &NUMTRIALS) != 1 || NUMTRIALS <= 0) {
|
||||
printf("Invalid trial count: %s should be > 0\n", argv[i]);
|
||||
return -1;
|
||||
}
|
||||
}
|
||||
else if (strcmp(argv[i], "-profile") == 0) {
|
||||
profileAPIs = 1;
|
||||
}
|
||||
else if (strcmp(argv[i], "-crossdev") == 0) {
|
||||
isCrossDevice = 1;
|
||||
}
|
||||
else if (strcmp(argv[i], "-width") == 0) {
|
||||
++i;
|
||||
if (sscanf(argv[i], "%d", &WIDTH) != 1 || (WIDTH <= 0)) {
|
||||
printf("Width should be greater than 0\n");
|
||||
return -1;
|
||||
}
|
||||
}
|
||||
else if (strcmp(argv[i], "-height") == 0) {
|
||||
++i;
|
||||
if (sscanf(argv[i], "%d", &HEIGHT) != 1 || (HEIGHT <= 0)) {
|
||||
printf("Width should be greater than 0\n");
|
||||
return -1;
|
||||
}
|
||||
}
|
||||
else if (0 == strcmp(&argv[i][1], "proctype")) {
|
||||
++i;
|
||||
if (!strcasecmp(argv[i], "prod")) {
|
||||
args->isProducer = 1;
|
||||
}
|
||||
else if (!strcasecmp(argv[i], "cons")) {
|
||||
args->isProducer = 0;
|
||||
}
|
||||
else {
|
||||
printf("%s: Bad Process Type: %s\n", __func__, argv[i]);
|
||||
return 1;
|
||||
}
|
||||
}
|
||||
else if (strcmp(argv[i], "-v") == 0) {
|
||||
verbose = 1;
|
||||
}
|
||||
else {
|
||||
printf("Unknown option: %s\n", argv[i]);
|
||||
return -1;
|
||||
}
|
||||
}
|
||||
|
||||
if (isCrossDevice) {
|
||||
int deviceCount = 0;
|
||||
|
||||
CUresult error_id = cuInit(0);
|
||||
if (error_id != CUDA_SUCCESS) {
|
||||
printf("cuInit(0) returned %d\n", error_id);
|
||||
printf("Result = FAIL\n");
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
|
||||
error_id = cuDeviceGetCount(&deviceCount);
|
||||
if (error_id != CUDA_SUCCESS) {
|
||||
printf("cuDeviceGetCount returned %d\n", (int)error_id);
|
||||
printf("Result = FAIL\n");
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
|
||||
int iGPUexists = 0;
|
||||
CUdevice dev;
|
||||
for (dev = 0; dev < deviceCount; ++dev) {
|
||||
int integrated = 0;
|
||||
CUresult error_result = cuDeviceGetAttribute(&integrated, CU_DEVICE_ATTRIBUTE_INTEGRATED, dev);
|
||||
|
||||
if (error_result != CUDA_SUCCESS) {
|
||||
printf("cuDeviceGetAttribute returned error : %d\n", (int)error_result);
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
|
||||
if (integrated) {
|
||||
iGPUexists = 1;
|
||||
}
|
||||
}
|
||||
|
||||
if (!iGPUexists) {
|
||||
printf("No Integrated GPU found in the system.\n");
|
||||
printf("-crossdev option is only supported on systems with an Integrated "
|
||||
"GPU and a Discrete GPU\n");
|
||||
printf("Waiving the execution\n");
|
||||
exit(EXIT_SUCCESS);
|
||||
}
|
||||
}
|
||||
|
||||
if (!eglSetupExtensions(isCrossDevice)) {
|
||||
printf("SetupExtentions failed \n");
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
#define MAX_EGL_DEVICES 4
|
||||
EGLDeviceEXT devices[MAX_EGL_DEVICES];
|
||||
EGLint numDevices = 0;
|
||||
EGLBoolean eglStatus = eglQueryDevicesEXT(MAX_EGL_DEVICES, devices, &numDevices);
|
||||
if (eglStatus != EGL_TRUE) {
|
||||
printf("Error querying EGL devices\n");
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
|
||||
if (numDevices == 0) {
|
||||
printf("No EGL devices found\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
exit(2); // EXIT_WAIVED
|
||||
}
|
||||
|
||||
int egl_device_id = 0;
|
||||
for (egl_device_id = 0; egl_device_id < numDevices; egl_device_id++) {
|
||||
EGLAttrib cuda_device;
|
||||
eglStatus = eglQueryDeviceAttribEXT(devices[egl_device_id], EGL_CUDA_DEVICE_NV, &cuda_device);
|
||||
if (eglStatus == EGL_TRUE) {
|
||||
break;
|
||||
}
|
||||
}
|
||||
|
||||
if (egl_device_id >= numDevices) {
|
||||
printf("No CUDA Capable EGL Device found.. Waiving execution\n");
|
||||
exit(2); // EXIT_WAIVED
|
||||
}
|
||||
|
||||
if (isCrossDevice) {
|
||||
if (numDevices == 1) {
|
||||
printf("Found only one EGL device, cannot setup cross GPU streams. "
|
||||
"Waiving\n");
|
||||
eglStatus = EGL_FALSE;
|
||||
exit(2); // EXIT_WAIVED
|
||||
}
|
||||
}
|
||||
|
||||
return 0;
|
||||
}
|
||||
|
||||
void launchProducer(TestArgs *args)
|
||||
{
|
||||
/* Cross-process creation of producer */
|
||||
char argsProducer[1024];
|
||||
char str[256];
|
||||
|
||||
strcpy(argsProducer, "./EGLStream_CUDA_CrossGPU -proctype prod ");
|
||||
|
||||
if (isCrossDevice) {
|
||||
sprintf(str, "-crossdev ");
|
||||
strcat(argsProducer, str);
|
||||
}
|
||||
|
||||
if (verbose) {
|
||||
sprintf(str, "-v ");
|
||||
strcat(argsProducer, str);
|
||||
}
|
||||
|
||||
/*Make the process run in bg*/
|
||||
strcat(argsProducer, "& ");
|
||||
|
||||
printf("\n%s: Crossproc Producer command: %s \n", __func__, argsProducer);
|
||||
|
||||
/*Create crossproc Producer*/
|
||||
system(argsProducer);
|
||||
|
||||
/*Enable crossproc Consumer in the same process */
|
||||
args->isProducer = 0;
|
||||
}
|
||||
|
||||
void printUsage(void)
|
||||
{
|
||||
printf("Usage:\n");
|
||||
printf(" -h Print this help message\n");
|
||||
printf(" -n n Exit after running n trials. Set to 10 by default\n");
|
||||
printf(" -profile Profile time taken by ReleaseAPI. Not set by default\n");
|
||||
printf(" -crossdev Run with producer on idgpu and consumer on dgpu\n");
|
||||
printf(" -dgpu (same as -crossdev, deprecated)\n");
|
||||
printf(" -v verbose output\n");
|
||||
}
|
||||
150
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/kernel.cu
Normal file
150
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/kernel.cu
Normal file
@@ -0,0 +1,150 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// DESCRIPTION: Simple CUDA consumer rendering sample app
|
||||
//
|
||||
|
||||
#include <EGL/egl.h>
|
||||
#include <EGL/eglext.h>
|
||||
#include <cuda.h>
|
||||
#include <cuda_runtime.h>
|
||||
#include <stdio.h>
|
||||
#include <string.h>
|
||||
|
||||
#include "eglstrm_common.h"
|
||||
|
||||
extern bool isCrossDevice;
|
||||
|
||||
__device__ static unsigned int numErrors = 0, errorFound = 0;
|
||||
__device__ void checkProducerDataGPU(char *data, int size, char expectedVal, int frameNumber)
|
||||
{
|
||||
if ((data[blockDim.x * blockIdx.x + threadIdx.x] != expectedVal) && (!errorFound)) {
|
||||
printf("Producer FOUND:%d expected: %d at %d for trial %d %d\n",
|
||||
data[blockDim.x * blockIdx.x + threadIdx.x],
|
||||
expectedVal,
|
||||
(blockDim.x * blockIdx.x + threadIdx.x),
|
||||
frameNumber,
|
||||
numErrors);
|
||||
numErrors++;
|
||||
errorFound = 1;
|
||||
return;
|
||||
}
|
||||
}
|
||||
|
||||
__device__ void checkConsumerDataGPU(char *data, int size, char expectedVal, int frameNumber)
|
||||
{
|
||||
if ((data[blockDim.x * blockIdx.x + threadIdx.x] != expectedVal) && (!errorFound)) {
|
||||
printf("Consumer FOUND:%d expected: %d at %d for trial %d %d\n",
|
||||
data[blockDim.x * blockIdx.x + threadIdx.x],
|
||||
expectedVal,
|
||||
(blockDim.x * blockIdx.x + threadIdx.x),
|
||||
frameNumber,
|
||||
numErrors);
|
||||
numErrors++;
|
||||
errorFound = 1;
|
||||
return;
|
||||
}
|
||||
}
|
||||
|
||||
__global__ void writeDataToBuffer(char *pSrc, char newVal) { pSrc[blockDim.x * blockIdx.x + threadIdx.x] = newVal; }
|
||||
|
||||
__global__ void testKernelConsumer(char *pSrc, char size, char expectedVal, char newVal, int frameNumber)
|
||||
{
|
||||
checkConsumerDataGPU(pSrc, size, expectedVal, frameNumber);
|
||||
}
|
||||
|
||||
__global__ void testKernelProducer(char *pSrc, char size, char expectedVal, char newVal, int frameNumber)
|
||||
{
|
||||
checkProducerDataGPU(pSrc, size, expectedVal, frameNumber);
|
||||
}
|
||||
__global__ void getNumErrors(int *numErr) { *numErr = numErrors; }
|
||||
|
||||
cudaError_t cudaProducer_filter(cudaStream_t pStream,
|
||||
char *pSrc,
|
||||
int width,
|
||||
int height,
|
||||
char expectedVal,
|
||||
char newVal,
|
||||
int frameNumber)
|
||||
{
|
||||
// in case where consumer is on dgpu and producer is on igpu when return is
|
||||
// called the frame is not copied back to igpu. So the consumer changes is not
|
||||
// visible to producer
|
||||
if (isCrossDevice == 0) {
|
||||
testKernelProducer<<<(width * height) / 1024, 1024, 1, pStream>>>(
|
||||
pSrc, width * height, expectedVal, newVal, frameNumber);
|
||||
}
|
||||
writeDataToBuffer<<<(width * height) / 1024, 1024, 1, pStream>>>(pSrc, newVal);
|
||||
return cudaSuccess;
|
||||
};
|
||||
|
||||
cudaError_t cudaConsumer_filter(cudaStream_t cStream,
|
||||
char *pSrc,
|
||||
int width,
|
||||
int height,
|
||||
char expectedVal,
|
||||
char newVal,
|
||||
int frameNumber)
|
||||
{
|
||||
testKernelConsumer<<<(width * height) / 1024, 1024, 1, cStream>>>(
|
||||
pSrc, width * height, expectedVal, newVal, frameNumber);
|
||||
writeDataToBuffer<<<(width * height) / 1024, 1024, 1, cStream>>>(pSrc, newVal);
|
||||
return cudaSuccess;
|
||||
};
|
||||
|
||||
cudaError_t cudaGetValueMismatch()
|
||||
{
|
||||
int numErr_h;
|
||||
int *numErr_d = NULL;
|
||||
cudaError_t err = cudaSuccess;
|
||||
err = cudaMalloc(&numErr_d, sizeof(int));
|
||||
if (err != cudaSuccess) {
|
||||
printf("Cuda Main: cudaMalloc failed with %s\n", cudaGetErrorString(err));
|
||||
return err;
|
||||
}
|
||||
getNumErrors<<<1, 1>>>(numErr_d);
|
||||
err = cudaDeviceSynchronize();
|
||||
if (err != cudaSuccess) {
|
||||
printf("Cuda Main: cudaDeviceSynchronize failed with %s\n", cudaGetErrorString(err));
|
||||
}
|
||||
err = cudaMemcpy(&numErr_h, numErr_d, sizeof(int), cudaMemcpyDeviceToHost);
|
||||
if (err != cudaSuccess) {
|
||||
printf("Cuda Main: cudaMemcpy failed with %s\n", cudaGetErrorString(err));
|
||||
cudaFree(numErr_d);
|
||||
return err;
|
||||
}
|
||||
err = cudaFree(numErr_d);
|
||||
if (err != cudaSuccess) {
|
||||
printf("Cuda Main: cudaFree failed with %s\n", cudaGetErrorString(err));
|
||||
return err;
|
||||
}
|
||||
if (numErr_h > 0) {
|
||||
return cudaErrorUnknown;
|
||||
}
|
||||
return cudaSuccess;
|
||||
}
|
||||
378
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/main.cpp
Normal file
378
cpp/2_Concepts_and_Techniques/EGLStream_CUDA_CrossGPU/main.cpp
Normal file
@@ -0,0 +1,378 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
#include "cudaEGL.h"
|
||||
#include "cuda_consumer.h"
|
||||
#include "cuda_producer.h"
|
||||
#include "eglstrm_common.h"
|
||||
#include "helper.h"
|
||||
#if defined(EXTENSION_LIST)
|
||||
EXTENSION_LIST(EXTLST_EXTERN)
|
||||
#endif
|
||||
|
||||
bool signal_stop = 0;
|
||||
extern bool verbose;
|
||||
|
||||
static void sig_handler(int sig)
|
||||
{
|
||||
signal_stop = 1;
|
||||
printf("Signal: %d\n", sig);
|
||||
}
|
||||
|
||||
void DoneCons(int consumerStatus, int send_fd)
|
||||
{
|
||||
EGLStreamFini();
|
||||
// get the final status from producer, combine and print
|
||||
int producerStatus = -1;
|
||||
if (-1 == recv(send_fd, (void *)&producerStatus, sizeof(int), 0)) {
|
||||
printf("%s: Cuda Consumer could not receive status from producer.\n", __func__);
|
||||
}
|
||||
close(send_fd);
|
||||
|
||||
if (producerStatus == 0 && consumerStatus == 0) {
|
||||
printf("&&&& EGLStream_CUDA_CrossGPU PASSED\n");
|
||||
exit(EXIT_SUCCESS);
|
||||
}
|
||||
else {
|
||||
printf("&&&& EGLStream_CUDA_CrossGPU FAILED\n");
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
}
|
||||
|
||||
void DoneProd(int producerStatus, int connect_fd)
|
||||
{
|
||||
EGLStreamFini();
|
||||
if (-1 == send(connect_fd, (void *)&producerStatus, sizeof(int), 0)) {
|
||||
printf("%s: Cuda Producer could not send status to consumer.\n", __func__);
|
||||
}
|
||||
close(connect_fd);
|
||||
if (producerStatus == 0) {
|
||||
exit(EXIT_SUCCESS);
|
||||
}
|
||||
else {
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
}
|
||||
|
||||
int WIDTH = 8192, HEIGHT = 8192;
|
||||
int main(int argc, char **argv)
|
||||
{
|
||||
TestArgs args = {0, false};
|
||||
CUresult curesult = CUDA_SUCCESS;
|
||||
unsigned int j = 0;
|
||||
cudaError_t err = cudaSuccess;
|
||||
EGLNativeFileDescriptorKHR fileDescriptor = EGL_NO_FILE_DESCRIPTOR_KHR;
|
||||
struct timespec start, end;
|
||||
CUeglFrame cudaEgl1, cudaEgl2;
|
||||
int consumerStatus = 0;
|
||||
int send_fd = -1;
|
||||
|
||||
if (parseCmdLine(argc, argv, &args) < 0) {
|
||||
printUsage();
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
printf("Width : %u, height: %u and iterations: %u\n", WIDTH, HEIGHT, NUMTRIALS);
|
||||
|
||||
if (!args.isProducer) // Consumer code
|
||||
{
|
||||
test_cuda_consumer_s cudaConsumer;
|
||||
memset(&cudaConsumer, 0, sizeof(test_cuda_consumer_s));
|
||||
cudaConsumer.profileAPI = profileAPIs;
|
||||
|
||||
// Hook up Ctrl-C handler
|
||||
signal(SIGINT, sig_handler);
|
||||
|
||||
if (!EGLStreamInit(isCrossDevice, !args.isProducer, EGL_NO_FILE_DESCRIPTOR_KHR)) {
|
||||
printf("EGLStream Init failed.\n");
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
cudaConsumer.cudaDevId = cudaDevIndexCons;
|
||||
curesult = cudaDeviceCreateConsumer(&cudaConsumer);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
cuCtxPushCurrent(cudaConsumer.context);
|
||||
|
||||
launchProducer(&args);
|
||||
|
||||
args.charCnt = WIDTH * HEIGHT * 4;
|
||||
|
||||
curesult = cuda_consumer_init(&cudaConsumer, &args);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer: Init failed, status: %d\n", curesult);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
cuCtxPopCurrent(&cudaConsumer.context);
|
||||
|
||||
send_fd = UnixSocketConnect(SOCK_PATH);
|
||||
if (-1 == send_fd) {
|
||||
printf("%s: Cuda Consumer cannot create socket %s\n", __func__, SOCK_PATH);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
cuCtxPushCurrent(cudaConsumer.context);
|
||||
cudaConsumer.eglStream = g_consumerEglStream;
|
||||
cudaConsumer.eglDisplay = g_consumerEglDisplay;
|
||||
|
||||
// Send the EGL stream FD to producer
|
||||
fileDescriptor = eglGetStreamFileDescriptorKHR(cudaConsumer.eglDisplay, cudaConsumer.eglStream);
|
||||
if (EGL_NO_FILE_DESCRIPTOR_KHR == fileDescriptor) {
|
||||
printf("%s: Cuda Consumer could not get EGL file descriptor.\n", __func__);
|
||||
eglDestroyStreamKHR(cudaConsumer.eglDisplay, cudaConsumer.eglStream);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
if (verbose)
|
||||
printf("%s: Cuda Consumer EGL stream FD obtained : %d.\n", __func__, fileDescriptor);
|
||||
|
||||
int res = -1;
|
||||
res = EGLStreamSendfd(send_fd, fileDescriptor);
|
||||
if (-1 == res) {
|
||||
printf("%s: Cuda Consumer could not send EGL file descriptor.\n", __func__);
|
||||
consumerStatus = -1;
|
||||
close(fileDescriptor);
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS != (curesult = cuEGLStreamConsumerConnect(&(cudaConsumer.cudaConn), cudaConsumer.eglStream))) {
|
||||
printf("FAILED Connect CUDA consumer with error %d\n", curesult);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
j = 0;
|
||||
for (j = 0; j < NUMTRIALS; j++) {
|
||||
curesult = cudaConsumerAcquireFrame(&cudaConsumer, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer Test failed for frame = %d\n", j + 1);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
curesult = cudaConsumerReleaseFrame(&cudaConsumer, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer Test failed for frame = %d\n", j + 1);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
curesult = cudaConsumerAcquireFrame(&cudaConsumer, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer Test failed for frame = %d\n", j + 1);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
curesult = cudaConsumerReleaseFrame(&cudaConsumer, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Consumer Test failed for frame = %d\n", j + 1);
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
}
|
||||
cuCtxSynchronize();
|
||||
close(fileDescriptor);
|
||||
err = cudaGetValueMismatch();
|
||||
if (err != cudaSuccess) {
|
||||
printf("Consumer: App failed with value mismatch\n");
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
EGLint streamState = 0;
|
||||
if (!eglQueryStreamKHR(cudaConsumer.eglDisplay, cudaConsumer.eglStream, EGL_STREAM_STATE_KHR, &streamState)) {
|
||||
printf("Main, eglQueryStreamKHR EGL_STREAM_STATE_KHR failed\n");
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
|
||||
if (streamState != EGL_STREAM_STATE_DISCONNECTED_KHR) {
|
||||
if (CUDA_SUCCESS != (curesult = cuda_consumer_Deinit(&cudaConsumer))) {
|
||||
printf("Consumer Disconnect FAILED.\n");
|
||||
consumerStatus = -1;
|
||||
DoneCons(consumerStatus, send_fd);
|
||||
}
|
||||
}
|
||||
}
|
||||
else // Producer
|
||||
{
|
||||
test_cuda_producer_s cudaProducer;
|
||||
memset(&cudaProducer, 0, sizeof(test_cuda_producer_s));
|
||||
cudaProducer.profileAPI = profileAPIs;
|
||||
int producerStatus = 0;
|
||||
|
||||
setenv("CUDA_EGL_PRODUCER_RETURN_WAIT_TIMEOUT", "1600", 0);
|
||||
|
||||
int connect_fd = -1;
|
||||
// Hook up Ctrl-C handler
|
||||
signal(SIGINT, sig_handler);
|
||||
|
||||
// Create connection to Consumer
|
||||
connect_fd = UnixSocketCreate(SOCK_PATH);
|
||||
if (-1 == connect_fd) {
|
||||
printf("%s: Cuda Producer could not create socket: %s.\n", __func__, SOCK_PATH);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
// Get the file descriptor of the stream from the consumer process
|
||||
// and re-create the EGL stream from it
|
||||
fileDescriptor = EGLStreamReceivefd(connect_fd);
|
||||
if (-1 == fileDescriptor) {
|
||||
printf("%s: Cuda Producer could not receive EGL file descriptor \n", __func__);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
if (!EGLStreamInit(isCrossDevice, 0, fileDescriptor)) {
|
||||
printf("EGLStream Init failed.\n");
|
||||
producerStatus = -1;
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
cudaProducer.eglDisplay = g_producerEglDisplay;
|
||||
cudaProducer.eglStream = g_producerEglStream;
|
||||
cudaProducer.cudaDevId = cudaDevIndexProd;
|
||||
|
||||
curesult = cudaDeviceCreateProducer(&cudaProducer);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
args.charCnt = WIDTH * HEIGHT * 4;
|
||||
cuCtxPushCurrent(cudaProducer.context);
|
||||
curesult = cudaProducerInit(&cudaProducer, &args);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer: Init failed, status: %d\n", curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
// wait for consumer to connect first
|
||||
int err = 0;
|
||||
int wait_loop = 0;
|
||||
EGLint streamState = 0;
|
||||
do {
|
||||
err =
|
||||
eglQueryStreamKHR(cudaProducer.eglDisplay, cudaProducer.eglStream, EGL_STREAM_STATE_KHR, &streamState);
|
||||
if ((0 != err) && (EGL_STREAM_STATE_CONNECTING_KHR != streamState)) {
|
||||
sleep(1);
|
||||
wait_loop++;
|
||||
}
|
||||
} while ((wait_loop < 10) && (0 != err) && (streamState != EGL_STREAM_STATE_CONNECTING_KHR));
|
||||
|
||||
if ((0 == err) || (wait_loop >= 10)) {
|
||||
printf("%s: Cuda Producer eglQueryStreamKHR EGL_STREAM_STATE_KHR failed.\n", __func__);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
if (CUDA_SUCCESS
|
||||
!= (curesult =
|
||||
cuEGLStreamProducerConnect(&(cudaProducer.cudaConn), cudaProducer.eglStream, WIDTH, HEIGHT))) {
|
||||
printf("Connect CUDA producer FAILED with error %d\n", curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
printf("main - Cuda Producer and Consumer Initialized.\n");
|
||||
|
||||
cudaProducerPrepareFrame(&cudaEgl1, cudaProducer.cudaPtr, args.charCnt);
|
||||
cudaProducerPrepareFrame(&cudaEgl2, cudaProducer.cudaPtr1, args.charCnt);
|
||||
|
||||
j = 0;
|
||||
for (j = 0; j < NUMTRIALS; j++) {
|
||||
curesult = cudaProducerPresentFrame(&cudaProducer, cudaEgl1, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer Test failed for frame = %d with cuda error:%d\n", j + 1, curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
curesult = cudaProducerPresentFrame(&cudaProducer, cudaEgl2, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer Test failed for frame = %d with cuda error:%d\n", j + 1, curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
curesult = cudaProducerReturnFrame(&cudaProducer, cudaEgl1, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer Test failed for frame = %d with cuda error:%d\n", j + 1, curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
curesult = cudaProducerReturnFrame(&cudaProducer, cudaEgl2, j);
|
||||
if (curesult != CUDA_SUCCESS) {
|
||||
printf("Cuda Producer Test failed for frame = %d with cuda error:%d\n", j + 1, curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
}
|
||||
|
||||
cuCtxSynchronize();
|
||||
err = cudaGetValueMismatch();
|
||||
if (err != cudaSuccess) {
|
||||
printf("Prod: App failed with value mismatch\n");
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
printf("Tear Down Start.....\n");
|
||||
if (!eglQueryStreamKHR(cudaProducer.eglDisplay, cudaProducer.eglStream, EGL_STREAM_STATE_KHR, &streamState)) {
|
||||
printf("Main, eglQueryStreamKHR EGL_STREAM_STATE_KHR failed\n");
|
||||
curesult = CUDA_ERROR_UNKNOWN;
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
|
||||
if (streamState != EGL_STREAM_STATE_DISCONNECTED_KHR) {
|
||||
if (CUDA_SUCCESS != (curesult = cudaProducerDeinit(&cudaProducer))) {
|
||||
printf("Producer Disconnect FAILED with %d\n", curesult);
|
||||
producerStatus = -1;
|
||||
DoneProd(producerStatus, connect_fd);
|
||||
}
|
||||
}
|
||||
unsetenv("CUDA_EGL_PRODUCER_RETURN_WAIT_TIMEOUT");
|
||||
}
|
||||
|
||||
return 0;
|
||||
}
|
||||
Reference in New Issue
Block a user